#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include "kd-tree.h"
#include "space.h"
#include "pool.h"
#include "genbench.h"

#define DEBUG(s...)	fprintf(stderr, s)

kdtree_pool *bank;
kdtree_space bank_space;



int main(int argc, char **argv, char **envp)
{
	kdtree_sphere bs;
	kdtree_pair_buffer *buf;
	int i;
	
	gb_init("Linear kd-tree unique sweep", argc, argv);
	
	kdtree_set_isospace(&bank_space,
		gb_min_x(), gb_max_x(),
		gb_min_y(), gb_max_y(),
		gb_min_z(), gb_max_z());
	

	while(1) {
		gb_timer_start(GB_COLLISION_PHASE);
		
		bank = kdtree_pool_new(gb_nobj(), gb_nobj());

		for (i = 0; i < gb_nobj(); i++) {
			gb_obj_coord(i, &bs.x, &bs.y, &bs.z);
			bs.r = gb_obj_radius(i);
			
			kdtree_pool_add_sphere(bank, &bs, (void *)(intptr_t)i);
		}
		
		kdtree_pool_build_tree(bank, &bank_space);
		gb_timer_start(GB_POSITION_UPDATE);
		kdtree_pool_sort_tree(bank);
		gb_timer_stop(GB_POSITION_UPDATE);
		kdtree_pool_sweep_tree(bank);
		

		if (bank->stats_out_of_bounds) {
			DEBUG("%d out-of-bounds spheres placed at root\n",
				bank->stats_out_of_bounds);
		}

		gb_tell_nodes(bank->nent);

		DEBUG("pair is %zd, ent is %zd\n",
			sizeof(kdtree_pair), sizeof(kdtree_entry));
		/* DEBUG("collisions: tree: %d, sphere: %d, unique: %d\n",
			bank->stats_tree_collisions,
			bank->stats_sphere_collisions,
			bank->stats_unique_collisions); */
		buf = bank->pair_buf;
		while(buf) {
			for (i = 0; i < buf->npairs; i++) {
				gb_collision(
					(intptr_t)KDTREE_PAIR_ELEMENT_A(
						bank, buf, i).pointer,
					(intptr_t)KDTREE_PAIR_ELEMENT_B(
						bank, buf, i).pointer);
			}
			buf = buf->next;
		}
		
		kdtree_pool_free(bank);
		
		gb_timer_stop(GB_COLLISION_PHASE);
		
		gb_next_step();
	}

	return(0);
}


