#ifndef __GENERIC_BENCHMARK_H
#define __GENERIC_BENCHMARK_H

// *******************************************************************
// Generic simulation model & object generator
// *******************************************************************
void gb_init(char *sim_name, int argc, char **argv);
void gb_collision(int n1, int n2);
void gb_next_step();

// accessors
int  gb_nobj();
void gb_obj_coord(int n, double *x, double *y, double *z);
void gb_obj_color(int n, double *r, double *g, double *b);
double gb_obj_radius(int n);
double gb_min_x();
double gb_max_x();
double gb_min_y();
double gb_max_y();
double gb_min_z();
double gb_max_z();

// *******************************************************************
// Simulation benchmarking
// *******************************************************************

/*
 Desired stats:
  Model:
   - frame number (simulation time), objects, mean and std. dev.
  Simulator:
   - positives, true positives, bounces
   - collision phase time, true collision handling time
   Octree:
    - AABB placement vs. pair sweep time
    - nodes
   Sweep and Prune:
    - position update time vs. pair sweep time
   Delaunay:
    - position update time vs. edge sweep time
   Quadratic: /
*/

/*
  Frame  Objects   mean x   mean y   mean z   dev x   dev y   dev z  bounces
   1000   100000    2.000    2.000 -100.000  50.000  50.000 100.000    10000

   Coll.    True+    Nodes  Coll. Time   Mid. Ph.     Handling  Update Time
  760000    10000   800000   77000.000  10000.000      100.000    76000.000
*/

#define GB_COLLISION_PHASE	0
#define GB_MIDDLE_PHASE		1
#define GB_COLLISION_HANDLING	2
#define GB_POSITION_UPDATE	3
void gb_timer_start(int id);
void gb_timer_stop(int id);

void gb_tell_nodes(int nodes);

// internal
void gb_timers_init();
void gb_timer_clear(int id);
double gb_timer_ms(int id);

// *******************************************************************
// Simulation benchmark & state output (internal)
// *******************************************************************
void gb_dump_stats(int with_header);
void gb_dump_state(int with_header);

// simulation file retreive
int  gb_load_sim(char *file_name);
int  gb_load_sim_file(FILE *f);
int  gb_load_frame();

#endif

